@@ -29,11 +29,12 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.54';
+$VERSION = '1.56';
 
 sub ConvertTimecode($);
 sub ProcessSGLT($$$);
 sub ProcessSLLT($$$);
+sub ProcessLucas($$$);
 
 # recognized RIFF variants
 my %riffType = (
@@ -142,7 +143,7 @@ my %code2charset = (
     0x64 => 'APICOM G.726 ADPCM',
     0x65 => 'APICOM G.722 ADPCM',
     0x66 => 'Microsoft DSAT', #6
-    0x67 => 'Micorsoft DSAT DISPLAY', #6
+    0x67 => 'Microsoft DSAT DISPLAY', #6
     0x69 => 'Voxware Byte Aligned', #7
     0x70 => 'Voxware AC8', #7
     0x71 => 'Voxware AC10', #7
@@ -339,7 +340,7 @@ my %code2charset = (
         these files, information is extracted from subsequent RIFF chunks as
         sub-documents, but the Duration is calculated for the full video.
     },
-    # (not 100% sure that the concatination technique mentioned above is valid - PH)
+    # (not 100% sure that the concatenation technique mentioned above is valid - PH)
    'fmt ' => {
         Name => 'AudioFormat',
         SubDirectory => { TagTable => 'Image::ExifTool::RIFF::AudioFormat' },
@@ -433,6 +434,14 @@ my %code2charset = (
             Condition => '$$valPt =~ /^PENTDigital Camera/',
             SubDirectory => { TagTable => 'Image::ExifTool::Pentax::Junk2' },
         },
+        {
+            Name => 'LucasJunk', # (Lucas LK-7900 Ace)
+            Condition => '$$valPt =~ /^0G(DA|PS)/',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::QuickTime::Stream',
+                ProcessProc => \&ProcessLucas,
+            },
+        },
         {
             Name => 'TextJunk',
             # try to interpret unknown junk as an ASCII string
@@ -473,17 +482,38 @@ my %code2charset = (
         Name => 'CharacterSet',
         SubDirectory => { TagTable => 'Image::ExifTool::RIFF::CSET' },
     },
+    # tx_ tags are generated based on the Codec used for the txts stream
+    tx_USER => {
+        Name => 'UserText',
+        SubDirectory => { TagTable => 'Image::ExifTool::RIFF::UserText' },
+    },
+    tx_Unknown => { # (untested)
+        Name => 'Text',
+        Notes => 'streamed text, extracted when the ExtractEmbedded option is used',
+    },
 #
 # WebP-specific tags
 #
-    EXIF => { # (WebP)
+    EXIF => [{ # (WebP)
         Name => 'EXIF',
+        Condition => '$$valPt =~ /^(II\x2a\0|MM\0\x2a)/',
         Notes => 'WebP files',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Exif::Main',
             ProcessProc => \&Image::ExifTool::ProcessTIFF,
         },
-    },
+    },{ # (WebP) - have also seen with "Exif\0\0" header - PH
+        Name => 'EXIF',
+        Condition => '$$valPt =~ /^Exif\0\0(II\x2a\0|MM\0\x2a)/',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Exif::Main',
+            ProcessProc => \&Image::ExifTool::ProcessTIFF,
+            Start => 6,
+        },
+    },{
+        Name => 'UnknownEXIF',
+        Binary => 1,
+    }],
    'XMP ' => { #14 (WebP)
         Name => 'XMP',
         Notes => 'WebP files',
@@ -974,6 +1004,12 @@ my %code2charset = (
             Condition => '$$self{RIFFStreamType} eq "vids"',
             SubDirectory => { TagTable => 'Image::ExifTool::BMP::Main' },
         },
+        {
+            Name => 'TextFormat',
+            Condition => '$$self{RIFFStreamType} eq "txts"',
+            Hidden => 1,
+            RawConv => '$self->Options("ExtractEmbedded") or $self->WarnOnce("Use ExtractEmbedded option to extract timed text"); undef',
+        },
     ],
 );
 
@@ -1004,7 +1040,7 @@ my %code2charset = (
     0 => {
         Name => 'StreamType',
         Format => 'string[4]',
-        RawConv => '$$self{RIFFStreamType} = $val',
+        RawConv => '$$self{RIFFStreamNum} = ($$self{RIFFStreamNum} || 0) + 1; $$self{RIFFStreamType} = $val',
         PrintConv => {
             auds => 'Audio',
             mids => 'MIDI',
@@ -1017,16 +1053,19 @@ my %code2charset = (
         {
             Name => 'AudioCodec',
             Condition => '$$self{RIFFStreamType} eq "auds"',
+            RawConv => '$$self{RIFFStreamCodec}[$$self{RIFFStreamNum}-1] = $val',
             Format => 'string[4]',
         },
         {
             Name => 'VideoCodec',
             Condition => '$$self{RIFFStreamType} eq "vids"',
+            RawConv => '$$self{RIFFStreamCodec}[$$self{RIFFStreamNum}-1] = $val',
             Format => 'string[4]',
         },
         {
             Name => 'Codec',
             Format => 'string[4]',
+            RawConv => '$$self{RIFFStreamCodec}[$$self{RIFFStreamNum}-1] = $val',
         },
     ],
   # 2 => 'StreamFlags',
@@ -1222,6 +1261,55 @@ my %code2charset = (
     },
 );
 
+# streamed USER txts written by Momento M6 dashcam (ref PH)
+%Image::ExifTool::RIFF::UserText = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 2 => 'Location' },
+    NOTES => q{
+        Tags decoded from the USER-format txts stream written by Momento M6 dashcam.
+        Extracted only if the ExtractEmbedded option is used.
+    },
+    # (little-endian)
+  #  0 - int32u: 32
+  #  4 - int32u: sample number (starting from unknown offset)
+  #  8 - int8u[4]: "w x y z" ? (w 0=front cam, 1=rear cam, z mostly 5-8)
+  # 12 - int8u[4]: "0 x 1 0" ? (x incrementing once per second)
+  # 16 - int8u[4]: "0 32 0 x" ?
+  # 20 - int32u: 100-150(mostly), 250-300(once per second)
+  # 24 - int8u[4]: "0 x y 0" ?
+    28 => { Name => 'GPSAltitude', Format => 'int32u', ValueConv => '$val / 10' }, # (NC)
+  # 32 - int32u: 0(mostly), 23(once per second)
+  # 36 - int32u: 0
+    40 => { Name => 'Accelerometer', Format => 'float[3]' },
+  # 52 - int32u: 1
+    56 => { Name => 'GPSSpeed',      Format => 'float' }, # km/h
+    60 => {
+        Name => 'GPSLatitude',
+        Format => 'float',
+        # Note: these values are unsigned and I don't know where the hemisphere is stored,
+        # but my only sample is from the U.S., so assume a positive latitude (for now)
+        ValueConv => 'my $deg = int($val / 100); $deg + ($val - $deg * 100) / 60',
+        PrintConv => 'Image::ExifTool::GPS::ToDMS($self, $val, 1, "N")',
+    },
+    64 => {
+        Name => 'GPSLongitude',
+        Format => 'float',
+        # Note: these values are unsigned and I don't know where the hemisphere is stored,
+        # but my only sample is from the U.S., so assume a negative longitude (for now)
+        ValueConv => 'my $deg = int($val / 100); -($deg + ($val - $deg * 100) / 60)',
+        PrintConv => 'Image::ExifTool::GPS::ToDMS($self, $val, 1, "E")',
+    },
+    68 => {
+        Name => 'GPSDateTime',
+        Description => 'GPS Date/Time',
+        Groups => { 2 => 'Time' },
+        Format => 'int32u',
+        ValueConv => 'ConvertUnixTime($val)',
+        # (likely local time, but clock seemed off by 3 hours in my sample)
+        PrintConv => '$self->ConvertDateTime($val)',
+    },
+);
+
 # WebP alpha info (ref 14)
 %Image::ExifTool::RIFF::ALPH = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
@@ -1621,6 +1709,105 @@ sub ProcessSLLT($$$)
     return 1;
 }
 
+#------------------------------------------------------------------------------
+# Process Lucas streaming GPS information (Lucas LK-7900 Ace) (ref PH)
+# Inputs: 0) ExifTool ref, 1) dirInfo ref, 2) tag table ref
+# Returns: 1 on success
+sub ProcessLucas($$$)
+{
+    my ($et, $dirInfo, $tagTbl) = @_;
+    my $dataPt = $$dirInfo{DataPt};
+    my $dataLen = length $$dataPt;
+
+    unless ($et->Options('ExtractEmbedded')) {
+        $et->Warn('Use ExtractEmbedded option to extract timed GPS', 3);
+        return 1;
+    }
+    my %recLen = (  # record lengths (not including 4-byte ID)
+        '0GDA' => 24,
+        '0GPS' => 48,
+    );
+    my ($date,$time,$lat,$lon,$alt,$spd,$sat,$dop,$ew,$ns);
+    $$et{SET_GROUP0} = $$et{SET_GROUP1} = 'RIFF';
+    while ($$dataPt =~ /(0GDA|0GPS)/g) {
+        my ($rec, $pos) = ($1, pos $$dataPt);
+        $pos + $recLen{$rec} > $dataLen and $et->Warn("Truncated $1 record"), last;
+        $$et{DOC_NUM} = ++$$et{DOC_COUNT};
+        # records start with int64u sample date/time in ms since 1970
+        $et->HandleTag($tagTbl, SampleDateTime => Get64u($dataPt, $pos) / 1000);
+        if ($rec eq '0GPS') {
+            my $len = Get32u($dataPt, $pos+8);
+            my $endPos = $pos + $recLen{$rec} + $len;
+            $endPos > $dataLen and $et->Warn('Truncated 0GPS record'), last;
+            my $buff = substr($$dataPt, $pos+$recLen{$rec}, $len);
+            while ($buff =~ /\$(GC|GA),(\d+),/g) {
+                my $p = pos $buff;
+                $time = $2;
+                if ($1 eq 'GC') {
+                    #     time   date   dist    ? sat dop alt  A
+                    # $GC,052350,180914,0000955,1,08,1.1,0017,,A*45\x0d\x0a\0
+                    if ($buff =~ /\G(\d+),\d*,\d*,(\d+),([-\d.]+),(\d+),\d*,A/g) {
+                        ($date,$sat,$dop,$alt) = ($1,$2,$3,$4);
+                    }
+                } else {
+                    #     time   A lat       lon        spd N W
+                    # $GA,052351,A,0949.6626,07635.4439,049,N,E,*4C\x0d\x0a\0
+                    if ($buff =~ /\GA,([\d.]+),([\d.]+),(\d+),([NS]),([EW])/g) {
+                        ($lat,$lon,$spd,$ns,$ew) = ($1,$2,$3,$4,$5,$6);
+                        # lat/long are in DDDMM.MMMM format
+                        my $deg = int($lat / 100);
+                        $lat = $deg + ($lat - $deg * 100) / 60;
+                        $deg = int($lon / 100);
+                        $lon = $deg + ($lon - $deg * 100) / 60;
+                        $lat *= -1 if $ns eq 'S';
+                        $lon *= -1 if $ew eq 'W';
+                    }
+                }
+                # look ahead to next NMEA-like sentence, and store the fix
+                # now only if the next sentence is not at the same time
+                if ($buff !~ /\$(GC|GA),$time,/g) {
+                    pos($$dataPt) = $endPos;
+                    if ($$dataPt !~ /\$(GC|GA),(\d+)/ or $1 ne $time) {
+                        $time =~ s/(\d{2})(\d{2})(\d{2})/$1:$2:$3Z/;
+                        if ($date) {
+                            $date =~ s/(\d{2})(\d{2})(\d{2})/20$3:$2:$1/;
+                            $et->HandleTag($tagTbl, GPSDateTime => "$date $time");
+                        } else {
+                            $et->HandleTag($tagTbl, GPSTimeStamp => $time);
+                        }
+                        if (defined $lat) {
+                            $et->HandleTag($tagTbl, GPSLatitude => $lat);
+                            $et->HandleTag($tagTbl, GPSLongitude => $lon);
+                            $et->HandleTag($tagTbl, GPSSpeed => $spd);
+                        }
+                        if (defined $alt) {
+                            $et->HandleTag($tagTbl, GPSAltitude => $alt);
+                            $et->HandleTag($tagTbl, GPSSatellites => $sat);
+                            $et->HandleTag($tagTbl, GPSDOP => $dop);
+                        }
+                        undef $lat;
+                        undef $alt;
+                    }
+                }
+                pos($buff) = $p;
+            }
+            $pos += $len;
+        } else { # this is an accelerometer (0GDA) record
+            # record has 4 more int32s values (the last is always 57 or 58 --
+            # maybe related to sample time in ms? -- not extracted)
+            my @acc = unpack('x'.($pos+8).'V3', $$dataPt);
+            # change to signed integer and divide by 256
+            map { $_ = $_ - 4294967296 if $_ >= 0x80000000; $_ /= 256 } @acc;
+            $et->HandleTag($tagTbl, Accelerometer => "@acc");
+        }
+        pos($$dataPt) = $pos + $recLen{$rec};
+    }
+    delete $$et{SET_GROUP0};
+    delete $$et{SET_GROUP1};
+    $$et{DOC_NUM} = 0;
+    return 1;
+}
+
 #------------------------------------------------------------------------------
 # Extract information from a RIFF file
 # Inputs: 0) ExifTool object reference, 1) DirInfo reference
@@ -1632,6 +1819,7 @@ sub ProcessRIFF($$)
     my ($buff, $buf2, $type, $mime, $err, $rf64);
     my $verbose = $et->Options('Verbose');
     my $unknown = $et->Options('Unknown');
+    my $ee = $et->Options('ExtractEmbedded');
 
     # verify this is a valid RIFF file
     return 0 unless $raf->Read($buff, 12) == 12;
@@ -1649,7 +1837,8 @@ sub ProcessRIFF($$)
     $mime = $riffMimeType{$type} if $type;
     $et->SetFileType($type, $mime);
     $$et{VALUE}{FileType} .= ' (RF64)' if $rf64;
-    $$et{RIFFStreamType} = '';    # initialize stream type
+    $$et{RIFFStreamType} = '';      # initialize stream type
+    $$et{RIFFStreamCodec} = [];     # initialize codec array
     SetByteOrder('II');
     my $tagTbl = GetTagTable('Image::ExifTool::RIFF::Main');
     my $pos = 12;
@@ -1688,14 +1877,20 @@ sub ProcessRIFF($$)
         # stop when we hit the audio data or AVI index or AVI movie data
         # --> no more because Adobe Bridge stores XMP after this!!
         # (so now we only do this on the FastScan option)
-        if (($tag eq 'data' or $tag eq 'idx1' or $tag eq 'LIST_movi') and
-            $et->Options('FastScan'))
+        if ($et->Options('FastScan') and ($tag eq 'data' or $tag eq 'idx1' or
+            ($tag eq 'LIST_movi' and not $ee)))
         {
             $et->VPrint(0, "(end of parsing)\n");
             last;
         }
         # RIFF chunks are padded to an even number of bytes
         my $len2 = $len + ($len & 0x01);
+        # change name of stream txts data depending on the Codec
+        if ($ee and $tag =~ /^(\d{2})tx$/) {
+            $tag = 'tx_' . ($$et{RIFFStreamCodec}[$1] || 'Unknown');
+            $tag = "tx_Unknown" unless defined $$tagTbl{$tag};
+            $$et{DOC_NUM} = ++$$et{DOC_COUNT};
+        }
         my $tagInfo = $$tagTbl{$tag};
         if ($tagInfo or (($verbose or $unknown) and $tag !~ /^(data|idx1|LIST_movi|RIFF)$/)) {
             $raf->Read($buff, $len2) == $len2 or $err=1, last;
@@ -1715,11 +1910,14 @@ sub ProcessRIFF($$)
                 delete $$et{SET_GROUP0};
                 delete $$et{SET_GROUP1};
             }
+            delete $$et{DOC_NUM} if $ee;
         } elsif ($tag eq 'RIFF') {
             # don't read into RIFF chunk (eg. concatenated video file)
             $raf->Read($buff, 4) == 4 or $err=1, last;
             # extract information from remaining file as an embedded file
             $$et{DOC_NUM} = ++$$et{DOC_COUNT}
+        } elsif ($tag eq 'LIST_movi' and $ee) {
+            next; # parse into movi chunk
         } else {
             if ($len > 0x7fffffff and not $et->Options('LargeFileSupport')) {
                 $et->Warn("Stopped parsing at large $tag chunk (LargeFileSupport not set)");
@@ -1754,7 +1952,7 @@ including AVI videos, WAV audio files and WEBP images.
 
 =head1 AUTHOR
 
-Copyright 2003-2019, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2021, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
